//
//  PCError.h
//  PayControl
//
//  Created by ООО СэйфТек on 06.10.15.
//  Copyright © 2015 ООО СэйфТек. All rights reserved.
//

#import <Foundation/Foundation.h>

/** Коды ошибок */

// TODO: Проверить и привести в соответсвие с ошибками на Андроид

typedef NS_ENUM(NSUInteger, PC_ERROR_CODE)
{
    /** Ошибки нет, т.е. все ОК */
    PC_ERROR_OK,
    /** Считанный QR код не может быть обработан */
    PC_ERROR_QR_INCORRECT,
    /** Ошибка получения сущностей из CoreDatа */
    PC_ERROR_PERFORM_FETCH_ERROR,
    /** XML не может быть прочитан */
    PC_ERROR_XML_PARSE_FAILED,
    /** Сканирование QR-кода было отменено пользователем и сканер был закрыт */
    PC_ERROR_QR_SCAN_CANCEL,
    /** RequestCode не соответствует */
    PC_ERROR_OTHER_REQUESTCODE,
    
    //PC_ERROR_QR_SCAN_OTHER_REQUESTCODE,
    
    /** Срок действия ключевой информации истек. Ее использование невозможно */
    PC_ERROR_KEY_DATE_EXPIRED,
    
    /** Необходимо объединение со второй частью ключа*/
    PC_ERROR_KEY_IS_NOT_FULL,
    /** Часть ключевой информации неверна */
    PC_ERROR_KEY_SECOND_PART_INCORRECT,
    /** Вторая часть ключа пустая */
    PC_ERROR_KEY_SECOND_PART_EMPTY,
    
    /** Хендлов быть не должно */
    PC_ERROR_KEY_HANDLE_VALID,
    /** Требуется ввод пароля */
    PC_ERROR_KEY_HANDLE_INVALID,
    /** Ключевая информация неверная */
    PC_ERROR_KEY_ENTIRE_NOT_VALID,
    /** Название ключа пустое */
    PC_ERROR_KEY_NAME_EMPTY,
    /** Пароль пустой */
    PC_ERROR_KEY_PASS_EMPTY,
    /** Введенный пароль неверный */
    PC_ERROR_KEY_PASS_INCORRECT,
    /** Ошибка сохранения ключевой информации */
    PC_ERROR_KEY_SAVE_FAILED,
    /** Пароль меньше требуемой длинны */
    PC_ERROR_KEY_PASS_LENGTH_INVALID,
    /** Ключ не найден */
    PC_ERROR_KEY_NOT_FOUND,
    /** Такое имя ключа уже существует */
    PC_ERROR_KEY_NAME_EXISTS,
    /** Ошибка при чтении PDF */
    PC_ERROR_PDF_VIEW,
    /** Достигнуто ограничение по максимальному количеству хранимых ключей */
    PC_ERROR_MAX_KEYS_COUNT_EXCEEDED,
    /** Произошла неизвестная ошибка (Все пропало шеф...=) */
    PC_ERROR_GENERAL,
    /** Ключевая пара уже существует */
    PC_ERROR_KEYPAIR_EXISTS,
    /** Операция не поддерживается */
    PC_ERROR_OPERATION_NOT_SUPPORTED,
    /** Ошибка при сохранении ключа */
    PC_ERROR_KEYS_ENCRYPTION_ERROR,
    /** Длина кода меньше минимальной */
    PC_ERROR_CONFIRMATION_CODE_LENGTH_SMALL,
    /** Длина кода больше максимальной */
    PC_ERROR_CONFIRMATION_CODE_LENGTH_BIG
};

/**
 Объект ошибки. Создание должно производиться с помощью методов класса: errorWithCode: или errorWithDescription: Созданный экземпляр является объектом для чтения и не подлежит модификации.
 */
@interface PCError : NSObject

/// ****************************************************************************
/// @name Методы класса
/// ****************************************************************************

/** Создание экземпляра объекта ошибки на основе кода ошибки - PC_ERROR_CODE 
 @param errorCode код ошибки
 */
+(nullable instancetype)errorWithCode:(PC_ERROR_CODE)errorCode;

/** Создание экземпляра объекта ошибки с произвольным тектовым описание при этом свойство errorCode будет равно PC_ERROR_GENERAL используется в специфических случаях когда необходимо получить системное описание ошибки из NSError 
 @param errorDescription строковое описание ошибки
 */
+(nullable instancetype)errorWithDescription:(NSString*)errorDescription;

/// ****************************************************************************
/// @name Свойства
/// ****************************************************************************

/** Код ошибки */
@property(nonatomic, readonly) PC_ERROR_CODE errorCode;

/** Строковое описание ошибки*/
@property(nonatomic, readonly, nullable) NSString *errorMessage;

@end



